#!/usr/bin/env python3
"""
Control middleware
"""
import contextlib
import shutil
import pathlib
import paramiko  # type: ignore
import pprint
import edi_997_inbound
import edi_944
import edi_947
import edi_846
import edi_867
import edi_997_outbound
import update_shandex_dashboard
import edi_943

THIS_DIRECTORY = pathlib.Path(__file__).parent
X12_SHANDEX_OUTGOING = THIS_DIRECTORY / "outgoing"
X12_SHANDEX_INCOMING = THIS_DIRECTORY / "incoming"


def main():
    """
    Do it!
    """
    #pick up files from Shandex
    retrieve_x12_edi_files_shandex()
    
    #process all EDIs that started with Shandex
    edi_997_inbound.main()
    edi_944.main()
    edi_947.main()
    #edi_846.main()
    edi_867.main()
    
    # process all EDIs that start with us
    edi_943.main()
    edi_997_outbound.main()
    
    #send them to Shandex
    send_x12_edi_files_shandex()#TODO production changes
    
    #update dashboard
    # update_shandex_dashboard.main()
    


SSH_DIRECTORY = THIS_DIRECTORY / "ssh"
SSH_KNOWN_HOSTS_FILE = str(SSH_DIRECTORY / "known_hosts")
SSH_KEY_FILENAME = str(SSH_DIRECTORY / "id_ed25519")


SHANDEX_SFTP_HOST = "ftp.shandex.com"
SHANDEX_SFTP_USERNAME = "Stash"
SHANDEX_SFTP_PASSWORD = "ST@Pass2024$$"


def send_x12_edi_files_shandex():
    """
    Connect to FTP & send files.
    """
    with paramiko.SSHClient() as ssh_client:
        ssh_client.load_system_host_keys()
        ssh_client.load_host_keys(SSH_KNOWN_HOSTS_FILE)
        ssh_client.set_missing_host_key_policy(paramiko.client.RejectPolicy)
        ssh_client.connect(
            hostname=SHANDEX_SFTP_HOST, username=SHANDEX_SFTP_USERNAME, password=SHANDEX_SFTP_PASSWORD
        )
        with ssh_client.open_sftp() as sftp_connection:
            sftp_connection.chdir("/Stash/Test/ToShandex") #TODO change to production folder
            for filename in X12_SHANDEX_OUTGOING.glob("*.edi"):
                sftp_connection.put(filename, str(filename.name))
                shutil.move(filename, X12_SHANDEX_OUTGOING / "archive" / filename.name)


def retrieve_x12_edi_files_shandex():
    with paramiko.SSHClient() as ssh_client:
        ssh_client.load_system_host_keys()
        ssh_client.load_host_keys(SSH_KNOWN_HOSTS_FILE)
        ssh_client.set_missing_host_key_policy(paramiko.client.RejectPolicy)
        ssh_client.connect(
            hostname=SHANDEX_SFTP_HOST, username=SHANDEX_SFTP_USERNAME, password=SHANDEX_SFTP_PASSWORD
        )
        with ssh_client.open_sftp() as sftp_connection:
            sftp_connection.chdir("/Stash/Test/FromShandex")
            for filename in sftp_connection.listdir():
                if filename.endswith(".edi"):
                    sftp_connection.get(filename, X12_SHANDEX_INCOMING / filename)
                    new_filename = f"/Stash/Test/FromShandex/Archive/{filename}"
                    sftp_connection.rename(filename, new_filename)


if __name__ == "__main__":
    main()
